!*****************************************************************************
                              program orbes
!*
!*                     Updated by Eduardo Mendes, Belo Horizonte, July 9, 2023
!*
!* To compile
!*
!* gfortran -Warray-bounds -fbounds-check OrbLiuYangEM.f90 -o OrbLiuYangEM.exe
!*
!*                    Warning: some parameters have to be specified at line 80
!*****************************************************************************
  implicit none
  integer, parameter :: mmax3 = 700, mmax2 = 60
  double precision :: ermmax, Dx, u, v, Xmin, Xmax
  integer :: compt, long2, Npt, Ncp, n, i, j, k, l, ll, ii, maux
  integer :: ik, in, is, iv, num_rows, mmax
  logical :: invert
  integer :: Pmax, status, ios
  character(len=10) :: argValue
  character(len=100) :: line
  ! Allocatable double precision
  double precision, allocatable :: p(:), q(:), y(:), Xc(:)
  double precision, allocatable :: x1(:), y1(:), er(:)
  ! Allocatable integers
  integer, allocatable :: a(:), posa(:)
  integer, allocatable :: m(:)
  integer, allocatable :: t(:), ss(:,:)
  
  ! Allocatable integers for other subrotuines
  integer, allocatable :: pm1(:), orb1(:,:), posa1(:)
  integer, allocatable :: ntab(:,:), tab(:), posa2(:)
  
  ! Allocatable double precision for other routines
  double precision, allocatable :: x2(:), y2(:)
  double precision, allocatable :: x11(:), y11(:)
  
! Get the command line argument
! call GET_COMMAND_ARGUMENT(1, argValue, status)

! Convert the argument to an integer
  write(*,*) 'Abscissa of the critical points are set at line 80 of the code...'
  write(*,*)
  write(*,*) 'What is the largest period of the orbits to extract?'
  read(*,*) Pmax
! read(argValue, *, iostat=status) Pmax
! if (status /= 0 .or. .not. (Pmax > 1)) then
!   ! Display error message and exit the program
!   write(*, *) "Error: Invalid argument. Please provide an integer greater than one."
!   stop
! end if
  
! Allocate variables
  allocate(orb1(mmax3, mmax2), posa1(mmax3), pm1(mmax3))
  allocate(x11(mmax3), y11(mmax3))

  allocate(x2(mmax3), y2(mmax3))
  allocate(ntab(mmax3, mmax2), tab(mmax3), posa2(mmax3))

! Internal variables
  allocate(x1(0:mmax2), y1(0:mmax2), er(0:mmax2))
  allocate(a(0:mmax2), posa(0:mmax2))
  
!****************************************************************************** 
  open(11,file='liuyangmap.dat',status='old')            ! Data file of the map
  num_rows = 0
  do
     read(11, '(A)', iostat=ios) line
     if (ios /= 0) exit                 ! Exit loop when end of file is reached
     num_rows = num_rows + 1
  end do
  ! Close the file
  close(11)

  ! Output the number of rows
  write(*, *) "Number of data points in the first-return map:", num_rows
  
  mmax = num_rows
  allocate(p(1:mmax), q(1:mmax), y(0:mmax), Xc(1:mmax))
  allocate(m(0:mmax))
  allocate(t(1:mmax), ss(1:mmax, 1:mmax))
!******************************************************************************
!*                                Warning: these parameter have to be specified
!*
      Ncp = 4                                      ! Numbers of critical points
      Xc(1) = -1.00                          ! X-coordinates of critical points
      Xc(2) = -0.50                       
      Xc(3) = +0.00                       
      Xc(4) = +0.50                       
      invert = .false.                   ! true when the map has to be inverted
!******************************************************************************
      Npt = mmax                                       ! Numbers of data points 
      ermmax = 0.100                      ! Largest error on the closest return

      open(11,file='liuyangmap.dat',status='old') 
      write(*,*) 'Extracting of orbits whose period is less than',Pmax+1,' ...'
      write(*,*) ' ' 
      write(*,*) 'Results are available in the file population.res'
      write(*,*) ' '

      Xmin = + 1.0E+5
      Xmax = - 1.0E+5
      if (invert) then
         do i = 1, Ncp
           Xc(i) = -Xc(i)
         enddo
      endif
      do i=1,Npt
        read(11,*) p(i), q(i)
        Xmin = min(Xmin, p(i))
        Xmax = max(Xmax, p(i))
        if (invert) then
          p(i) = -p(i)   
          q(i) = -q(i)   
        endif
      enddo
      close(11)
      Dx = abs(Xmax - Xmin)
    
      n=0
      compt=0
 13   n = n+1
      if (n.gt.Pmax) goto 22
      write(*,*) 'periode',n
      Npt = Npt - n
      do i=1,Npt
        u = p(i+n) - p(i)
        v = q(i+n) - q(i)
        y(i) = dsqrt(u*u + v*v) / Dx
      enddo
      m(1) = 1
      do i=2,Npt
        m(i) = i
        j = i - 1
        do while(j.gt.0.and.y(m(j)).gt.y(m(j+1)))
           maux = m(j)
           m(j) = m(j+1)
           m(j+1) = maux
           j = j - 1
        enddo
      enddo
      k = 0
      is = 0
      in = 0
   11 iv = 0
      l = 1000
      if (l.eq.0) goto 16 
      ll = l
  17  j = k + 1
      k = k + l
      if (k.gt.Npt) k = Npt
      do i=j,k
        in = in + 1
        do l=0,n-1
          ii = 0
          do while(p(m(i)+l).gt.Xc(ii+1).and.ii.lt.Ncp)
            ii = ii + 1
          enddo
          t(l+1) = ii
        enddo
        do ik=1,is
		  if (ik >= mmax) then
			  write(*, '(A, I0, A)') " Number of ", mmax, " data points exceeded"
			  goto 22
		  end if
          do l=1,n
            if(t(l).ne.ss(ik,l)) goto 14
          enddo
          goto 15
  14      Continue
        enddo
        iv = iv + 1
        do l=0,n-1
          ii = 0
          do while(p(m(i)+l).gt.Xc(ii+1).and.ii.lt.Ncp)
            ii = ii + 1
          enddo
          a(l)=ii
          x1(l)=p(m(i)+l)
          y1(l)=q(m(i)+l)
          posa(l)=m(i)+l
          er(l)=y(m(i)+l)
        enddo

       if (er(0).le.ermmax) then
           
            compt=compt+1
  		    if (compt >= mmax3) then
  			  write(*, '(A, I0, A)') " Number of ", mmax3, " orbits exceeded"
  			  goto 22 
  		    end if
            pm1(compt)=n
            do ll=0,n-1
              orb1(compt,ll+1)=a(ll)
            enddo
            x11(compt)=x1(0)
            y11(compt)=y1(0)
            posa1(compt)=posa(0)
       endif

        do l=1,n
          t(l+n) = t(l)
        enddo
        do ik = is+1,is+n
          if (ik >= mmax) then
            write(*, '(A, I0, A)') " Number of ", mmax, " data points exceeded"
            goto 22
          end if
          do l=1,n+n-1
            t(l) = t(l+1)
          enddo
          do l=1,n
            ss(ik,l) = t(l)
          enddo
        enddo
        is = is + n
  15    Continue
      enddo
      if (k.eq.Npt) goto 12 
      if (iv.lt.ll) then
        l = ll - iv
        goto 17
      endif
      goto 11
  12  Continue
  16  print*
      goto 13
  22  continue
  call circ(compt, long2, tab, ntab, x2, y2, posa2,&
                        &  pm1, orb1, x11, y11, posa1, mmax2, mmax3)
  write(*, *) 'Pruning completed:', compt,'periodic orbits were extracted'
  call ord(long2, tab, ntab, x2, y2, posa2, mmax2, mmax3)
  deallocate(x2, y2)
  deallocate(ntab, tab, posa2)
  deallocate(orb1, posa1, pm1)
  deallocate(x11,y11)
  deallocate(p, q, y, Xc)
  deallocate(x1, y1, er)
  deallocate(a, posa)
  deallocate(m)
  deallocate(t, ss)

end program orbes
!**************************************************************************
subroutine circ(long3, long1, tab, ntab, x1, y1, posa,&
	                        & pm1, orb1, x11, y11, posa1, nPmax, ndim)
!**************************************************************************
  implicit none
  !integer, parameter :: nPmax = 60, ndim = 510
  integer :: i, j, k, Pmax, nPmax, ndim
  integer :: mmmax, w, l, m, a
  integer :: long3, long1
  integer, allocatable :: orbi(:,:,:), finorb(:,:)
  integer :: orb1(ndim, nPmax), posa1(ndim), pm1(ndim)
  double precision :: x11(ndim), y11(ndim)
  double precision :: x1(ndim), y1(ndim)
  integer :: ntab(ndim, nPmax), tab(ndim), posa(ndim)
  !common /der1/ pm1, orb1, x11, y11, posa1
  !common /der2/ tab, ntab, x1, y1, posa
  
  allocate(orbi(nPmax, ndim, nPmax*2), finorb(ndim, nPmax))
  
  long1 = 0
  w = 0
        
  do i = 1, long3
    Pmax = pm1(i)
    if (Pmax == 1) then
      long1 = long1 + 1
      tab(long1) = Pmax
              
      do j = 1, Pmax
        ntab(long1, j) = orb1(i, j)
      end do
      x1(long1) = x11(i)
      y1(long1) = y11(i)
              
      posa(long1) = posa1(i)
      goto 10
    endif

    orbi(1, i, 1:Pmax) = orb1(i, 1:Pmax)
    orbi(1, i, Pmax+1:Pmax*2) = orb1(i, 1:Pmax)

    do k = 2, Pmax
      orbi(k, i, 1:Pmax) = orbi(1, i, k:k+Pmax-1)
    end do

    l = 1
    do while (l < Pmax)
      m = l + 1
      do while (m <= Pmax)
        j = 1
        k = 0
        a = 0
        do while ((orbi(l, i, j) == orbi(m, i, j)) .and. (j <= Pmax))
          a = orbi(l, i, j) + a
          k = k + 1
          j = j + 1
        enddo
        if (k == Pmax) goto 10
        if (mod(a, 2) == 0) then
          if (orbi(l, i, j) > orbi(m, i, j)) then
            mmmax = l
          else
            mmmax = m
            goto 5
          endif
        else
          if (orbi(l, i, j) > orbi(m, i, j)) then
            mmmax = m
            goto 5
          else
            mmmax = l
          endif
        endif
        m = m + 1
      enddo
      if (mmmax == l) goto 7
5     l = l + 1
    enddo
7   w = w + 1
    do j = 1, Pmax
      finorb(w, j) = orbi(mmmax, i, j)
    enddo
    long1 = long1 + 1
    tab(long1) = Pmax
          
    do j = 1, Pmax
      ntab(long1, j) = orbi(mmmax, i, j)
    enddo
    x1(long1) = x11(i)
    y1(long1) = y11(i)
    posa(long1) = posa1(i)
10  continue
  enddo
  deallocate(orbi, finorb)  
end subroutine

!*************************************************************************
  subroutine ord(long3, tab, ntab, x1, y1, posa, mmax2, mmax1)
!*************************************************************************
  implicit none
  !integer, parameter :: mmax1 = 510, mmax2 = 60
  integer :: i, j, k, l, long3, mmax1, mmax2
  double precision :: x1(mmax1), y1(mmax1)
  integer :: ntab(mmax1, mmax2), tab(mmax1), posa(mmax1)
  !integer :: tab(mmax1), ntab(mmax1, mmax2), posa(mmax1)
  !double precision :: x1(mmax1), y1(mmax1)
  integer, allocatable :: atab(:), btab(:)
  integer :: a, b, som, mmmax
  double precision :: c, d
  character(len=256) :: my_fmt
  integer :: max_value
  !common /der2/ tab, ntab, x1, y1, posa
 
  allocate(atab(mmax2), btab(mmax2))
 
  open(16,file='population.res')
       
   do i = 1, long3
     j = tab(i) + 1
     ntab(i, j:mmax2) = ntab(i, j-tab(i):mmax2-tab(i))
   end do
       i=1
19     do while(i.lt.long3)
         j=i+1
          do while(j.le.long3)
          l=1

          som=0
          do while (ntab(i,l).eq.ntab(j,l))
             som=ntab(i,l)+som
             l=l+1
          enddo
          if (mod(som,2).eq.0) then
                if (ntab(i,l).gt.ntab(j,l)) then
                  mmmax=i
                else
                  mmmax=j
                endif
          else
                if (ntab(i,l).gt.ntab(j,l)) then
                  mmmax=j
                else
                  mmmax=i
               endif
          endif
          if (mmmax.eq.i) then
             do k=1,mmax2
             atab(k)=ntab(i,k)
             btab(k)=ntab(j,k)
             ntab(i,k)=btab(k)
             ntab(j,k)=atab(k)
             enddo
             a=tab(i)
             b=tab(j)
             tab(i)=b
             tab(j)=a
             a=posa(i)
             b=posa(j)
             posa(i)=b
             posa(j)=a
             c=x1(i)
             d=x1(j)
             x1(i)=d
             x1(j)=c
             c=y1(i)
             d=y1(j)
             y1(i)=d
             y1(j)=c
             goto 19
          endif
          j=j+1
          enddo
        i=i+1
        enddo
		max_value = maxval(tab(1:long3))
		!write(*,*) "Maximum value of the first",long3," elements:", max_value
		do i=1,long3
			if (tab(i).eq.1) then
				write(my_fmt,'("(i2,",i0,"x,d20.12,1x,d20.12,4x,i5)")') max_value
			else
				write(my_fmt,'("(i2,",i0,"i1,",i0,"x,d20.12,1x,d20.12,4x,i5)")') (tab(i)-1),(max_value-tab(i)+1)
			end if
			write(16,my_fmt) (ntab(i,j),j=1,tab(i)),x1(i),y1(i),posa(i)
		end do
		
        close(16)
		
		deallocate(atab, btab)
		
end subroutine
